@@ -1,4 +1,4 @@
-# Copyright (c) 2018 Micro Focus or one of its affiliates.
+# Copyright (c) 2018-2019 Micro Focus or one of its affiliates.
 # Copyright (c) 2018 Uber Technologies, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -38,6 +38,8 @@ from __future__ import print_function, division, absolute_import
 
 import re
 
+from .vertica.mixins.notice_response_attr import _NoticeResponseAttrMixin
+
 
 #############################################
 # dbapi errors
@@ -95,6 +97,8 @@ class TimedOutError(OperationalError):
 class ConnectionError(DatabaseError):
     pass
 
+class KerberosError(ConnectionError):
+    pass
 
 class SSLNotSupported(ConnectionError):
     pass
@@ -108,7 +112,18 @@ class EmptyQueryError(ProgrammingError):
     pass
 
 
-class QueryError(ProgrammingError):
+class QueryError(_NoticeResponseAttrMixin, ProgrammingError):
+    """
+    An error occurred associated with a query had been issued.
+
+    This error is perhaps the most commonly encountered error type, associated
+    with failures during query execution, invalid SQL statements, and more.
+
+    Much of the details of the error are available as properties, including the
+    SQL statement that had been issued (`sql`).
+    """
+
+
     def __init__(self, error_response, sql):
         self.error_response = error_response
         self.sql = sql
@@ -122,6 +137,13 @@ class QueryError(ProgrammingError):
         else:
             return ''
 
+    @property
+    def _notice_attrs(self):
+        # provided for _NoticeResponseAttrMixin
+        if self.error_response is None:
+            return {}
+        return self.error_response._notice_attrs or {}
+
     @classmethod
     def from_error_response(cls, error_response, sql):
         klass = QUERY_ERROR_CLASSES.get(error_response.sqlstate, None)
@@ -187,18 +209,18 @@ class LostConnectivityFailure(QueryError):
 
 
 QUERY_ERROR_CLASSES = {
-    b'55V03': LockFailure,
-    b'53000': InsufficientResources,
-    b'53200': OutOfMemory,
-    b'42601': VerticaSyntaxError,
-    b'3F000': MissingSchema,
-    b'42V01': MissingRelation,
-    b'42703': MissingColumn,
-    b'22V04': CopyRejected,
-    b'42501': PermissionDenied,
-    b'22007': InvalidDatetimeFormat,
-    b'42710': DuplicateObject,
-    b'57014': QueryCanceled,
-    b'08006': ConnectionFailure,
-    b'V1001': LostConnectivityFailure
+    '55V03': LockFailure,
+    '53000': InsufficientResources,
+    '53200': OutOfMemory,
+    '42601': VerticaSyntaxError,
+    '3F000': MissingSchema,
+    '42V01': MissingRelation,
+    '42703': MissingColumn,
+    '22V04': CopyRejected,
+    '42501': PermissionDenied,
+    '22007': InvalidDatetimeFormat,
+    '42710': DuplicateObject,
+    '57014': QueryCanceled,
+    '08006': ConnectionFailure,
+    'V1001': LostConnectivityFailure
 }
