@@ -36,6 +36,7 @@
 from __future__ import print_function, division, absolute_import
 
 from decimal import Decimal
+from uuid import UUID
 
 from .base import VerticaPythonIntegrationTestCase
 
@@ -59,5 +60,12 @@ class TypeTestCase(VerticaPythonIntegrationTestCase):
         res = self._query_and_fetchone(query)
         self.assertEqual(res[0], value)
 
+    def test_uuid_query(self):
+        self.require_protocol_at_least(3 << 16 | 8)
+        value = UUID('00010203-0405-0607-0809-0a0b0c0d0e0f')
+        query = "SELECT '{0}'::uuid".format(value)
+        res = self._query_and_fetchone(query)
+        self.assertEqual(res[0], value)
 
-exec(TypeTestCase.createPrepStmtClass())
\ No newline at end of file
+
+exec(TypeTestCase.createPrepStmtClass())
