@@ -36,17 +36,23 @@
 
 from __future__ import print_function, division, absolute_import
 
+import base64
 import logging
 import socket
 import ssl
 import getpass
 import uuid
 from struct import unpack
-from collections import deque
+from collections import deque, namedtuple
 
 # noinspection PyCompatibility,PyUnresolvedReferences
 from builtins import str
-from six import raise_from, string_types, integer_types
+from six import raise_from, string_types, integer_types, PY2
+
+if PY2:
+    from urlparse import urlparse, parse_qs
+else:
+    from urllib.parse import urlparse, parse_qs
 
 import vertica_python
 from .. import errors
@@ -59,9 +65,15 @@ from ..vertica.log import VerticaLogging
 DEFAULT_HOST = 'localhost'
 DEFAULT_PORT = 5433
 DEFAULT_PASSWORD = ''
+DEFAULT_BACKUP_SERVER_NODE = []
+DEFAULT_KRB_SERVICE_NAME = 'vertica'
 DEFAULT_LOG_LEVEL = logging.WARNING
 DEFAULT_LOG_PATH = 'vertica_python.log'
-ASCII = 'ascii'
+try:
+    DEFAULT_USER = getpass.getuser()
+except Exception as e:
+    DEFAULT_USER = None
+    print("WARN: Cannot get the login user name: {}".format(str(e)))
 
 
 def connect(**kwargs):
@@ -69,13 +81,54 @@ def connect(**kwargs):
     return Connection(kwargs)
 
 
+def parse_dsn(dsn):
+    """Parse connection string into a dictionary of keywords and values.
+       Connection string format:
+           vertica://<user>:<password>@<host>:<port>/<database>?k1=v1&k2=v2&...
+    """
+    url = urlparse(dsn)
+    if url.scheme != 'vertica':
+        raise ValueError("Only vertica:// scheme is supported.")
+
+    # Ignore blank/invalid values
+    result = {k: v for k, v in (
+        ('host', url.hostname),
+        ('port', url.port),
+        ('user', url.username),
+        ('password', url.password),
+        ('database', url.path[1:])) if v
+    }
+    for key, value in parse_qs(url.query).items():
+        if key == 'backup_server_node':
+            continue
+        elif key in ('connection_load_balance', 'use_prepared_statements', 'ssl'):
+            lower = value[-1].lower()
+            if lower in ('true', 'on', '1'):
+                result[key] = True
+            elif lower in ('false', 'off', '0'):
+                result[key] = False
+        elif key == 'connection_timeout':
+            result[key] = float(value[-1])
+        elif key == 'log_level' and value[-1].isdigit():
+            result[key] = int(value[-1])
+        else:
+            result[key] = value[-1]
+
+    return result
+
+_AddressEntry = namedtuple('_AddressEntry', ['host', 'resolved', 'data'])
+
 class _AddressList(object):
     def __init__(self, host, port, backup_nodes, logger):
         """Creates a new deque with the primary host first, followed by any backup hosts"""
 
         self._logger = logger
 
-        # Format of items in deque: (host, port, is_dns_resolved)
+        # Items in address_deque are _AddressEntry values.
+        #   host is the original hostname/ip, used by SSL option check_hostname
+        #   - when resolved is False, data is port
+        #   - when resolved is True, data is the 5-tuple from socket.getaddrinfo
+        # This allows for lazy resolution. Seek peek() for more.
         self.address_deque = deque()
 
         # load primary host into address_deque
@@ -126,41 +179,49 @@ class _AddressList(object):
             self._logger.error(err_msg)
             raise ValueError(err_msg)
 
-        self.address_deque.append((host, port, False))
+        self.address_deque.append(_AddressEntry(host=host, resolved=False, data=port))
 
     def push(self, host, port):
-        self.address_deque.appendleft((host, port, False))
+        self.address_deque.appendleft(_AddressEntry(host=host, resolved=False, data=port))
 
     def pop(self):
         self.address_deque.popleft()
 
     def peek(self):
-        # do lazy DNS resolution, return the leftmost DNS-resolved address
+        # do lazy DNS resolution, returning the leftmost socket.getaddrinfo result
         if len(self.address_deque) == 0:
             return None
 
         while len(self.address_deque) > 0:
-            host, port, is_dns_resolved = self.address_deque[0]
-            if is_dns_resolved:
-                # return a resolved address
-                self._logger.debug('Peek at address list: {0}'.format(list(self.address_deque)))
-                return (host, port)
+            self._logger.debug('Peek at address list: {0}'.format(list(self.address_deque)))
+            entry = self.address_deque[0]
+            if entry.resolved:
+                # return a resolved sockaddrinfo
+                return entry.data
             else:
                 # DNS resolve a single host name to multiple IP addresses
                 self.address_deque.popleft()
+                host, port = entry.host, entry.data
                 try:
-                    resolved_hosts = socket.getaddrinfo(host, port, socket.AF_INET, socket.SOCK_STREAM)
+                    resolved_hosts = socket.getaddrinfo(host, port, 0, socket.SOCK_STREAM)
                 except Exception as e:
                     self._logger.warning('Error resolving host "{0}" on port {1}: {2}'.format(host, port, e))
                     continue
 
-                # add resolved IP addresses to deque
-                for res in reversed(resolved_hosts):
-                    family, socktype, proto, canonname, sockaddr = res
-                    self.address_deque.appendleft((sockaddr[0], sockaddr[1], True))
-
+                # add resolved addrinfo (AF_INET and AF_INET6 only) to deque
+                for addrinfo in reversed(resolved_hosts):
+                    if addrinfo[0] in (socket.AF_INET, socket.AF_INET6):
+                        self.address_deque.appendleft(_AddressEntry(
+                            host=host, resolved=True, data=addrinfo))
         return None
 
+    def peek_host(self):
+        # returning the leftmost host result
+        self._logger.debug('Peek host at address list: {0}'.format(list(self.address_deque)))
+        if len(self.address_deque) == 0:
+            return None
+        return self.address_deque[0].host
+
 
 def _generate_session_label():
     return '{type}-{version}-{id}'.format(
@@ -180,7 +241,9 @@ class Connection(object):
         self.socket = None
 
         options = options or {}
-        self.options = {key: value for key, value in options.items() if value is not None}
+        self.options = parse_dsn(options['dsn']) if 'dsn' in options else {}
+        self.options.update({key: value for key, value in options.items() \
+                             if key != 'dsn' and value is not None})
 
         # Set up connection logger
         logger_name = 'vertica_{0}_{1}'.format(id(self), str(uuid.uuid4())) # must be a unique value
@@ -192,24 +255,29 @@ class Connection(object):
         else:
             self.options.setdefault('log_level', DEFAULT_LOG_LEVEL)
             self.options.setdefault('log_path', DEFAULT_LOG_PATH)
-            VerticaLogging.setup_file_logging(logger_name, self.options['log_path'],
-                                              self.options['log_level'], id(self))
+            VerticaLogging.setup_logging(logger_name, self.options['log_path'],
+                                         self.options['log_level'], id(self))
 
         self.options.setdefault('host', DEFAULT_HOST)
         self.options.setdefault('port', DEFAULT_PORT)
         if 'user' not in self.options:
-            try:
-                self.options['user'] = getpass.getuser()
-            except Exception as e:
-                self._logger.error(
-                    "Failed to set default value for connection 'user': {}".format(str(e)))
-                raise KeyError('Connection option "user" is required')
+            if DEFAULT_USER:
+                self.options['user'] = DEFAULT_USER
+            else:
+                msg = 'Connection option "user" is required'
+                self._logger.error(msg)
+                raise KeyError(msg)
         self.options.setdefault('database', self.options['user'])
         self.options.setdefault('password', DEFAULT_PASSWORD)
         self.options.setdefault('session_label', _generate_session_label())
+        self.options.setdefault('backup_server_node', DEFAULT_BACKUP_SERVER_NODE)
+        self.options.setdefault('kerberos_service_name', DEFAULT_KRB_SERVICE_NAME)
+        # Kerberos authentication hostname defaults to the host value here so
+        # the correct value cannot be overwritten by load balancing or failover
+        self.options.setdefault('kerberos_host_name', self.options['host'])
 
         self.address_list = _AddressList(self.options['host'], self.options['port'],
-                                         self.options.get('backup_server_node', []), self._logger)
+                                         self.options['backup_server_node'], self._logger)
 
         # we only support one cursor per connection
         self.options.setdefault('unicode_error', None)
@@ -227,11 +295,17 @@ class Connection(object):
         self.startup_connection()
         self._logger.info('Connection is ready')
 
+    #############################################
+    # supporting `with` statements
+    #############################################
     def __enter__(self):
         return self
 
     def __exit__(self, type_, value, traceback):
         try:
+            # get the transaction status
+            if not self.closed():
+                self.cursor().flush_to_query_ready()
             # if there's no outstanding transaction, we can simply close the connection
             if self.transaction_status in (None, 'in_transaction'):
                 return
@@ -253,12 +327,6 @@ class Connection(object):
         finally:
             self.close_socket()
 
-    def cancel(self):
-        if self.closed():
-            raise errors.ConnectionError('Connection is closed')
-
-        self.write(CancelRequest(backend_pid=self.backend_pid, backend_key=self.backend_key))
-
     def commit(self):
         if self.closed():
             raise errors.ConnectionError('Connection is closed')
@@ -284,6 +352,40 @@ class Connection(object):
         self._cursor.cursor_type = cursor_type
         return self._cursor
 
+    #############################################
+    # non-dbapi methods
+    #############################################
+    def cancel(self):
+        """Cancel the current database operation. This can be called from a
+           different thread than the one currently executing a database operation.
+        """
+        if self.closed():
+            raise errors.ConnectionError('Connection is closed')
+        self._logger.info('Canceling the current database operation')
+        # Must create a new socket connection to the server
+        temp_socket = self.establish_socket_connection(self.address_list)
+        self.write(CancelRequest(self.backend_pid, self.backend_key), temp_socket)
+        temp_socket.close()
+
+        self._logger.info('Cancel request issued')
+
+    def opened(self):
+        return (self.socket is not None
+                and self.backend_pid is not None
+                and self.transaction_status is not None)
+
+    def closed(self):
+        return not self.opened()
+
+    def __str__(self):
+        safe_options = {key: value for key, value in self.options.items() if key != 'password'}
+
+        s1 = "<Vertica.Connection:{0} parameters={1} backend_pid={2}, ".format(
+            id(self), self.parameters, self.backend_pid)
+        s2 = "backend_key={0}, transaction_status={1}, socket={2}, options={3}>".format(
+            self.backend_key, self.transaction_status, self.socket, safe_options)
+        return ''.join([s1, s2])
+
     #############################################
     # internal
     #############################################
@@ -295,14 +397,14 @@ class Connection(object):
         self.transaction_status = None
         self.socket = None
         self.address_list = _AddressList(self.options['host'], self.options['port'],
-                                         self.options.get('backup_server_node', []), self._logger)
+                                         self.options['backup_server_node'], self._logger)
 
     def _socket(self):
         if self.socket:
             return self.socket
 
         # the initial establishment of the client connection
-        raw_socket = self.establish_connection()
+        raw_socket = self.establish_socket_connection(self.address_list)
 
         # enable load balancing
         load_balance_options = self.options.get('connection_load_balance')
@@ -320,9 +422,9 @@ class Connection(object):
         self.socket = raw_socket
         return self.socket
 
-    def create_socket(self):
-        # Address family IPv6 (socket.AF_INET6) is not supported
-        raw_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+    def create_socket(self, family):
+        """Create a TCP socket object"""
+        raw_socket = socket.socket(family, socket.SOCK_STREAM)
         raw_socket.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
         connection_timeout = self.options.get('connection_timeout')
         if connection_timeout is not None:
@@ -336,19 +438,20 @@ class Connection(object):
         raw_socket.sendall(messages.LoadBalanceRequest().get_message())
         response = raw_socket.recv(1)
 
-        if response in (b'Y', 'Y'):
+        if response == b'Y':
             size = unpack('!I', raw_socket.recv(4))[0]
             if size < 4:
                 err_msg = "Bad message size: {0}".format(size)
                 self._logger.error(err_msg)
                 raise errors.MessageError(err_msg)
-            res = BackendMessage.from_type(type_=response, data=raw_socket.recv(size-4))
+            res = BackendMessage.from_type(type_=response, data=raw_socket.recv(size - 4))
             self._logger.debug('<= %s', res)
             host = res.get_host()
             port = res.get_port()
             self._logger.info('Load balancing to host "{0}" on port {1}'.format(host, port))
 
-            socket_host, socket_port = raw_socket.getpeername()
+            peer = raw_socket.getpeername()
+            socket_host, socket_port = peer[0], peer[1]
             if host == socket_host and port == socket_port:
                 self._logger.info('Already connecting to host "{0}" on port {1}. Ignore load balancing.'.format(host, port))
                 return raw_socket
@@ -357,7 +460,7 @@ class Connection(object):
             # will leave the originally-specified host as the first failover possibility.
             self.address_list.push(host, port)
             raw_socket.close()
-            raw_socket = self.establish_connection()
+            raw_socket = self.establish_socket_connection(self.address_list)
         else:
             self._logger.debug('<= LoadBalanceResponse: %s', response)
             self._logger.warning("Load balancing requested but not supported by server")
@@ -365,50 +468,62 @@ class Connection(object):
         return raw_socket
 
     def enable_ssl(self, raw_socket, ssl_options):
-        from ssl import CertificateError, SSLError
         # Send SSL request and read server response
         self._logger.debug('=> %s', messages.SslRequest())
         raw_socket.sendall(messages.SslRequest().get_message())
         response = raw_socket.recv(1)
         self._logger.debug('<= SslResponse: %s', response)
-        if response in ('S', b'S'):
+        if response == b'S':
             self._logger.info('Enabling SSL')
             try:
                 if isinstance(ssl_options, ssl.SSLContext):
-                    host, port = raw_socket.getpeername()
-                    raw_socket = ssl_options.wrap_socket(raw_socket, server_hostname=host)
+                    server_host = self.address_list.peek_host()
+                    if server_host is None:   # This should not happen
+                        msg = 'Cannot get the connected server host while enabling SSL'
+                        self._logger.error(msg)
+                        raise errors.ConnectionError(msg)
+                    raw_socket = ssl_options.wrap_socket(raw_socket, server_hostname=server_host)
                 else:
                     raw_socket = ssl.wrap_socket(raw_socket)
-            except CertificateError as e:
-                raise_from(errors.ConnectionError, e)
-            except SSLError as e:
-                raise_from(errors.ConnectionError, e)
+            except ssl.CertificateError as e:
+                raise_from(errors.ConnectionError(str(e)), e)
+            except ssl.SSLError as e:
+                raise_from(errors.ConnectionError(str(e)), e)
         else:
             err_msg = "SSL requested but not supported by server"
             self._logger.error(err_msg)
             raise errors.SSLNotSupported(err_msg)
         return raw_socket
 
-    def establish_connection(self):
-        addr = self.address_list.peek()
+    def establish_socket_connection(self, address_list):
+        """Given a list of database node addresses, establish the socket
+           connection to the database server. Return a connected socket object.
+        """
+        addrinfo = address_list.peek()
         raw_socket = None
         last_exception = None
 
         # Failover: loop to try all addresses
-        while addr:
+        while addrinfo:
+            (family, socktype, proto, canonname, sockaddr) = addrinfo
             last_exception = None
-            host, port = addr
+
+            # _AddressList filters all addrs to AF_INET and AF_INET6, which both
+            # have host and port as values 0, 1 in the sockaddr tuple.
+            host = sockaddr[0]
+            port = sockaddr[1]
 
             self._logger.info('Establishing connection to host "{0}" on port {1}'.format(host, port))
+
             try:
-                raw_socket = self.create_socket()
-                raw_socket.connect((host, port))
+                raw_socket = self.create_socket(family)
+                raw_socket.connect(sockaddr)
                 break
             except Exception as e:
                 self._logger.info('Failed to connect to host "{0}" on port {1}: {2}'.format(host, port, e))
                 last_exception = e
-                self.address_list.pop()
-                addr = self.address_list.peek()
+                address_list.pop()
+                addrinfo = address_list.peek()
                 raw_socket.close()
 
         # all of the addresses failed
@@ -422,24 +537,16 @@ class Connection(object):
     def ssl(self):
         return self.socket is not None and isinstance(self.socket, ssl.SSLSocket)
 
-    def opened(self):
-        return (self.socket is not None
-                and self.backend_pid is not None
-                and self.transaction_status is not None)
-
-    def closed(self):
-        return not self.opened()
-
-    def write(self, message):
+    def write(self, message, vsocket=None):
         if not isinstance(message, FrontendMessage):
             raise TypeError("invalid message: ({0})".format(message))
-
-        sock = self._socket()
+        if vsocket is None:
+            vsocket = self._socket()
         self._logger.debug('=> %s', message)
         try:
             for data in message.fetch_message():
                 try:
-                    sock.sendall(data)
+                    vsocket.sendall(data)
                 except Exception:
                     self._logger.error("couldn't send message")
                     raise
@@ -469,6 +576,8 @@ class Connection(object):
 
     def handle_asynchronous_message(self, message):
         if isinstance(message, messages.ParameterStatus):
+            if message.name == 'protocol_version':
+                message.value = int(message.value)
             self.parameters[message.name] = message.value
         elif (isinstance(message, messages.NoticeResponse) and
              not isinstance(message, messages.ErrorResponse)):
@@ -487,11 +596,14 @@ class Connection(object):
                 message = BackendMessage.from_type(type_, self.read_bytes(size - 4))
                 self._logger.debug('<= %s', message)
                 self.handle_asynchronous_message(message)
+                # handle transaction status
+                if isinstance(message, messages.ReadyForQuery):
+                    self.transaction_status = message.transaction_status
             except (SystemError, IOError) as e:
                 self.close_socket()
                 # noinspection PyTypeChecker
                 self._logger.error(e)
-                raise_from(errors.ConnectionError, e)
+                raise_from(errors.ConnectionError(str(e)), e)
             if not self.is_asynchronous_message(message):
                 break
         return message
@@ -516,39 +628,6 @@ class Connection(object):
             self._logger.error(msg)
             raise errors.MessageError(msg)
 
-    def process_message(self, message):
-        if isinstance(message, messages.ErrorResponse):
-            raise errors.ConnectionError(message.error_message())
-        elif isinstance(message, messages.BackendKeyData):
-            self.backend_pid = message.pid
-            self.backend_key = message.key
-        elif isinstance(message, messages.ReadyForQuery):
-            self.transaction_status = message.transaction_status
-        elif isinstance(message, messages.CommandComplete):
-            # TODO: I'm not ever seeing this actually returned by vertica...
-            # if vertica returns a row count, set the rowcount attribute in cursor
-            # if hasattr(message, 'rows'):
-            #     self.cursor.rowcount = message.rows
-            pass
-        elif isinstance(message, messages.EmptyQueryResponse):
-            pass
-        elif isinstance(message, messages.CopyInResponse):
-            pass
-        else:
-            raise errors.MessageError("Unhandled message: {0}".format(message))
-
-        # set last message
-        self._cursor._message = message
-
-    def __str__(self):
-        safe_options = {key: value for key, value in self.options.items() if key != 'password'}
-
-        s1 = "<Vertica.Connection:{0} parameters={1} backend_pid={2}, ".format(
-            id(self), self.parameters, self.backend_pid)
-        s2 = "backend_key={0}, transaction_status={1}, socket={2}, options={3}>".format(
-            self.backend_key, self.transaction_status, self.socket, safe_options)
-        return ''.join([s1, s2])
-
     def read_bytes(self, n):
         results = bytes()
         while len(results) < n:
@@ -558,20 +637,89 @@ class Connection(object):
             results += bytes_
         return results
 
-    def startup_connection(self):
-        # This doesn't handle Unicode usernames or passwords
-        user = self.options['user'].encode(ASCII)
-        database = self.options['database'].encode(ASCII)
-        password = self.options['password'].encode(ASCII)
-        session_label = self.options['session_label'].encode(ASCII)
+    def send_GSS_response_and_receive_challenge(self, response):       
+        # Send the GSS response data to the vertica server
+        token = base64.b64decode(response)
+        self.write(messages.Password(token, messages.Authentication.GSS))
+        # Receive the challenge from the vertica server
+        message = self.read_expected_message(messages.Authentication)
+        if message.code != messages.Authentication.GSS_CONTINUE:
+            msg = ('Received unexpected message type: Authentication(type={}).'
+                   ' Expected type: Authentication(type={})'.format(
+                   message.code, messages.Authentication.GSS_CONTINUE))
+            self._logger.error(msg)
+            raise errors.MessageError(msg)
+        return message.auth_data
 
-        self.write(messages.Startup(user, database, session_label))
+    def make_GSS_authentication(self):
+        try:
+            import kerberos
+        except ImportError as e:
+            raise errors.ConnectionError("{}\nCannot make a Kerberos "
+                "authentication because no Kerberos package is installed. "
+                "Get it with 'pip install kerberos'.".format(str(e)))
+
+        # Set GSS flags
+        gssflag = (kerberos.GSS_C_DELEG_FLAG | kerberos.GSS_C_MUTUAL_FLAG |
+                   kerberos.GSS_C_SEQUENCE_FLAG | kerberos.GSS_C_REPLAY_FLAG)
+
+        # Generate the GSS-style service principal name
+        service_principal = "{}@{}".format(self.options['kerberos_service_name'],
+                                           self.options['kerberos_host_name'])
+
+        # Initializes a context object with a service principal
+        self._logger.info('Initializing a context for GSSAPI client-side '
+            'authentication with service principal {}'.format(service_principal))
+        try:
+            result, context = kerberos.authGSSClientInit(service_principal, gssflags=gssflag)
+        except kerberos.GSSError as err:
+            msg = "GSSAPI initialization error: {}".format(str(err))
+            self._logger.error(msg)
+            raise errors.KerberosError(msg)
+        if result != kerberos.AUTH_GSS_COMPLETE:
+            msg = ('Failed to initialize a context for GSSAPI client-side '
+                   'authentication with service principal {}'.format(service_principal))
+            self._logger.error(msg)
+            raise errors.KerberosError(msg)
+
+        # Processes GSSAPI client-side steps
+        try:
+            challenge = b''
+            while True:
+                self._logger.info('Processing a single GSSAPI client-side step')
+                challenge = base64.b64encode(challenge).decode("utf-8")
+                result = kerberos.authGSSClientStep(context, challenge)
+
+                if result == kerberos.AUTH_GSS_COMPLETE:
+                    self._logger.info('Result: GSSAPI step complete')
+                    break
+                elif result == kerberos.AUTH_GSS_CONTINUE:
+                    self._logger.info('Result: GSSAPI step continuation')
+                    # Get the response from the last successful GSSAPI client-side step
+                    response = kerberos.authGSSClientResponse(context)
+                    challenge = self.send_GSS_response_and_receive_challenge(response)
+                else:
+                    msg = "GSSAPI client-side step error status {}".format(result)
+                    self._logger.error(msg)
+                    raise errors.KerberosError(msg)
+        except kerberos.GSSError as err:
+            msg = "GSSAPI client-side step error: {}".format(str(err))
+            self._logger.error(msg)
+            raise errors.KerberosError(msg)
+
+    def startup_connection(self):
+        user = self.options['user']
+        database = self.options['database']
+        session_label = self.options['session_label']
+        os_user_name = DEFAULT_USER if DEFAULT_USER else ''
+        password = self.options['password']
+
+        self.write(messages.Startup(user, database, session_label, os_user_name))
 
         while True:
             message = self.read_message()
 
             if isinstance(message, messages.Authentication):
-                # Password message isn't right format ("incomplete message from client")
                 if message.code == messages.Authentication.OK:
                     self._logger.info("User {} successfully authenticated"
                         .format(self.options['user']))
@@ -582,13 +730,22 @@ class Connection(object):
                 elif message.code == messages.Authentication.PASSWORD_GRACE:
                     self._logger.warning('The password for user {} will expire soon.'
                         ' Please consider changing it.'.format(self.options['user']))
+                elif message.code == messages.Authentication.GSS:
+                    self.make_GSS_authentication()
                 else:
                     self.write(messages.Password(password, message.code,
                                                  {'user': user,
                                                   'salt': getattr(message, 'salt', None),
                                                   'usersalt': getattr(message, 'usersalt', None)}))
-            else:
-                self.process_message(message)
-
-            if isinstance(message, messages.ReadyForQuery):
+            elif isinstance(message, messages.BackendKeyData):
+                self.backend_pid = message.pid
+                self.backend_key = message.key
+            elif isinstance(message, messages.ReadyForQuery):
                 break
+            elif isinstance(message, messages.ErrorResponse):
+                self._logger.error(message.error_message())
+                raise errors.ConnectionError(message.error_message())
+            else:
+                msg = "Received unexpected startup message: {0}".format(message)
+                self._logger.error(msg)
+                raise errors.MessageError(msg)
