@@ -36,25 +36,29 @@
 
 from __future__ import print_function, division, absolute_import
 
-import os, errno
+import errno
+import os
 import logging
 
+
 class VerticaLogging(object):
 
     @classmethod
-    def setup_file_logging(cls, logger_name, logfile, log_level=logging.INFO, context=''):
+    def setup_logging(cls, logger_name, logfile, log_level=logging.INFO, context=''):
         logger = logging.getLogger(logger_name)
-        formatter = logging.Formatter(
-            fmt=('%(asctime)s.%(msecs)03d [%(module)s] '
-                 '{}/%(process)d:0x%(thread)x <%(levelname)s> '
-                 '%(message)s'.format(context)),
-            datefmt='%Y-%m-%d %H:%M:%S')
-        cls.ensure_dir_exists(logfile)
-        file_handler = logging.FileHandler(logfile)
-        file_handler.setFormatter(formatter)
-        logger.addHandler(file_handler)
         logger.setLevel(log_level)
 
+        if logfile:
+            formatter = logging.Formatter(
+                fmt=('%(asctime)s.%(msecs)03d [%(module)s] '
+                     '{}/%(process)d:0x%(thread)x <%(levelname)s> '
+                     '%(message)s'.format(context)),
+                datefmt='%Y-%m-%d %H:%M:%S')
+            cls.ensure_dir_exists(logfile)
+            file_handler = logging.FileHandler(logfile, encoding='utf-8')
+            file_handler.setFormatter(formatter)
+            logger.addHandler(file_handler)
+
     @classmethod
     def ensure_dir_exists(cls, filepath):
         """Ensure that a directory exists
