@@ -48,7 +48,6 @@ from struct import unpack
 
 from ..message import BackendMessage
 
-UTF_8 = 'utf-8'
 
 class CommandComplete(BackendMessage):
     message_id = b'C'
@@ -56,9 +55,10 @@ class CommandComplete(BackendMessage):
     def __init__(self, data):
         BackendMessage.__init__(self)
         data = unpack('{0}sx'.format(len(data) - 1), data)[0]
-        self.command_tag = data.decode(UTF_8)
+        self.command_tag = data.decode('utf-8')
 
     def __str__(self):
         return 'CommandComplete: command_tag = "{}"'.format(self.command_tag)
 
+
 BackendMessage.register(CommandComplete)
