@@ -37,9 +37,9 @@
 CommandDescription message -- part of the response to a Describe request message.
 
 This response informs the client about the type of command being executed.
-If the command is a parameterized INSERT statement, the copy_rewrite field may 
-include a semantically-equivalent COPY STDIN statement. Clients can choose to 
-run this statement instead to achieve better performance when loading many 
+If the command is a parameterized INSERT statement, the copy_rewrite field may
+include a semantically-equivalent COPY STDIN statement. Clients can choose to
+run this statement instead to achieve better performance when loading many
 batches of parameters.
 """
 
@@ -49,7 +49,6 @@ from struct import unpack
 
 from ..message import BackendMessage
 
-UTF_8 = 'utf-8'
 
 class CommandDescription(BackendMessage):
     message_id = b'm'
@@ -59,13 +58,14 @@ class CommandDescription(BackendMessage):
         pos = data.find(b'\x00')
         unpacked = unpack("!{0}sxH{1}sx".format(pos, len(data) - pos - 4), data)
 
-        self.command_tag = unpacked[0].decode(UTF_8)
+        self.command_tag = unpacked[0].decode('utf-8')
         self.has_copy_rewrite = (unpacked[1] == 1)
-        self.copy_rewrite = unpacked[2].decode(UTF_8)
+        self.copy_rewrite = unpacked[2].decode('utf-8')
 
     def __str__(self):
         return ('CommandDescription: command_tag = "{}", has_copy_rewrite = {},'
                 ' copy_rewrite = "{}"'.format(
                     self.command_tag, self.has_copy_rewrite, self.copy_rewrite))
 
+
 BackendMessage.register(CommandDescription)
