@@ -38,6 +38,7 @@ from __future__ import print_function, division, absolute_import
 from ..message import BackendMessage
 from struct import unpack
 
+
 class LoadBalanceResponse(BackendMessage):
     message_id = b'Y'
 
@@ -45,7 +46,7 @@ class LoadBalanceResponse(BackendMessage):
         BackendMessage.__init__(self)
         unpacked = unpack('!I{0}sx'.format(len(data) - 5), data)
         self.port = unpacked[0]
-        self.host = unpacked[1]
+        self.host = unpacked[1].decode('utf-8')
 
     def get_port(self):
         return self.port
@@ -56,4 +57,5 @@ class LoadBalanceResponse(BackendMessage):
     def __str__(self):
         return "LoadBalanceResponse: host={}, port={}".format(self.host, self.port)
 
+
 BackendMessage.register(LoadBalanceResponse)
