@@ -1,4 +1,4 @@
-# Copyright (c) 2018 Micro Focus or one of its affiliates.
+# Copyright (c) 2018-2019 Micro Focus or one of its affiliates.
 # Copyright (c) 2018 Uber Technologies, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -35,68 +35,75 @@
 
 from __future__ import print_function, division, absolute_import
 
+import six
 from struct import unpack_from
 
+from ...mixins.notice_response_attr import _NoticeResponseAttrMixin
 from ..message import BackendMessage
 
 
-class NoticeResponse(BackendMessage):
+class NoticeResponse(_NoticeResponseAttrMixin, BackendMessage):
     message_id = b'N'
 
-    FIELDS_DEFINITIONS = [
-        {'type': b'q', 'name': "Internal Query", 'method': 'internal_query'},
-        {'type': b'S', 'name': "Severity", 'method': 'severity'},
-        {'type': b'M', 'name': "Message", 'method': 'message'},
-        {'type': b'C', 'name': "Sqlstate", 'method': 'sqlstate'},
-        {'type': b'D', 'name': "Detail", 'method': 'detail'},
-        {'type': b'H', 'name': "Hint", 'method': 'hint'},
-        {'type': b'P', 'name': "Position", 'method': 'position'},
-        {'type': b'W', 'name': "Where", 'method': 'where'},
-        {'type': b'p', 'name': "Internal Position", 'method': 'internal_position'},
-        {'type': b'R', 'name': "Routine", 'method': 'routine'},
-        {'type': b'F', 'name': "File", 'method': 'file'},
-        {'type': b'L', 'name': "Line", 'method': 'line'},
-        {'type': b'V', 'name': "Error Code", 'method': 'error_code'}
-    ]
-
     def __init__(self, data):
         BackendMessage.__init__(self)
-        self.values = {}
+        # `_notice_attrs` is required by _NoticeResponseAttrMixin and also used
+        # by QueryError
+        self._notice_attrs = NoticeResponse._unpack_data(data)
+
+    def error_message(self):
+        return ', '.join([
+            "{0}: {1}".format(name, value)
+            for (name, value) in six.iteritems(self.values)
+        ])
+
+    def __str__(self):
+        return "NoticeResponse: {}".format(self.error_message())
+
+    @property
+    def values(self):
+        """
+        A mapping of server-provided values describing this notice.
+
+        The keys of this mapping are user-facing strings. The contents of any
+        given NoticeResponse can vary based on the context or version of
+        Vertica.
+
+        For access to specific values, the appropriate property getter is
+        recommended.
+
+        Example return value:
+
+        ```
+            {
+                'Severity': 'ERROR',
+                'Message': 'Syntax error at or near "foobar"',
+                'Sqlstate': '42601',
+                'Position': '1',
+                'Routine': 'base_yyerror',
+                'File': '/data/.../vertica/Parser/scan.l',
+                'Line': '1043',
+                'Error Code': '4856'
+            }
+        ```
+        """
+        return self._get_labeled_values()
+
+    @staticmethod
+    def _unpack_data(data):
+        data_mapping = {}
 
         pos = 0
         while pos < len(data) - 1:
             null_byte = data.find(b'\x00', pos)
 
-            # This will probably work
             unpacked = unpack_from('c{0}sx'.format(null_byte - 1 - pos), data, pos)
             key = unpacked[0]
             value = unpacked[1]
+            data_mapping[key] = value.decode('utf-8')
 
-            self.values[self.fields()[key]] = value
             pos += (len(value) + 2)
 
-        # May want to break out into a function at some point
-        for field_def in self.FIELDS_DEFINITIONS:
-            if self.values.get(field_def['name'], None) is not None:
-                setattr(self, field_def['method'], self.values[field_def['name']])
-
-    def fields(self):
-        # was FIELDS before
-        # TODO verify that it doesn't break anything
-        pairs = []
-        for field in self.FIELDS_DEFINITIONS:
-            pairs.append((field['type'], field['name']))
-        return dict(pairs)
-
-    def error_message(self):
-        ordered = []
-        for field in self.FIELDS_DEFINITIONS:
-            if self.values.get(field['name']) is not None:
-                ordered.append("{0}: {1}".format(field['name'], self.values[field['name']]))
-        return ', '.join(ordered)
-
-    def __str__(self):
-        return "NoticeResponse: {}".format(self.error_message())
-
+        return data_mapping
 
 BackendMessage.register(NoticeResponse)
