@@ -33,52 +33,71 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
+"""
+Startup message
+
+To begin a session, the frontend opens a connection to the backend and sends a
+Startup message.
+"""
+
 from __future__ import print_function, division, absolute_import
 
 import platform
 import os
-import getpass
 from struct import pack
 
 # noinspection PyUnresolvedReferences,PyCompatibility
-from builtins import str
 
 import vertica_python
 from ..message import BulkFrontendMessage
 
-ASCII = 'ascii'
-
 
 class Startup(BulkFrontendMessage):
     message_id = None
 
-    def __init__(self, user, database, session_label, options=None):
+    def __init__(self, user, database, session_label, os_user_name):
         BulkFrontendMessage.__init__(self)
 
-        self._user = user
-        self._database = database
-        self._session_label = session_label
-        self._options = options
-        self._type = b'vertica-python'
-        self._version = vertica_python.__version__.encode(ASCII)
-        self._platform = platform.platform().encode(ASCII)
-        self._pid = '{0}'.format(os.getpid()).encode(ASCII)
-        self._os_user_name = getpass.getuser().encode(ASCII)
+        try:
+            os_platform = platform.platform()
+        except Exception as e:
+            os_platform = ''
+            print("WARN: Cannot get the OS info: {}".format(str(e)))
+
+        try:
+            pid = str(os.getpid())
+        except Exception as e:
+            pid = '0'
+            print("WARN: Cannot get the process ID: {}".format(str(e)))
+
+        self.parameters = {
+            b'user': user,
+            b'database': database,
+            b'client_label': session_label,
+            b'client_type': 'vertica-python',
+            b'client_version': vertica_python.__version__,
+            b'client_os': os_platform,
+            b'client_os_user_name': os_user_name,
+            b'client_pid': pid,
+        }
 
     def read_bytes(self):
-        bytes_ = pack('!I', vertica_python.PROTOCOL_VERSION)
-        if self._user is not None:
-            bytes_ += pack('4sx{0}sx'.format(len(self._user)), b'user', self._user)
-        if self._database is not None:
-            bytes_ += pack('8sx{0}sx'.format(len(self._database)), b'database', self._database)
-        if self._options is not None:
-            bytes_ += pack('7sx{0}sx'.format(len(self._options)), b'options', self._options)
-        bytes_ += pack('12sx{0}sx'.format(len(self._session_label)), b'client_label', self._session_label)
-        bytes_ += pack('11sx{0}sx'.format(len(self._type)), b'client_type', self._type)
-        bytes_ += pack('14sx{0}sx'.format(len(self._version)), b'client_version', self._version)
-        bytes_ += pack('9sx{0}sx'.format(len(self._platform)), b'client_os', self._platform)
-        bytes_ += pack('19sx{0}sx'.format(len(self._os_user_name)), b'client_os_user_name', self._os_user_name)
-        bytes_ += pack('10sx{0}sx'.format(len(self._pid)), b'client_pid', self._pid)
-        bytes_ += pack('x')
+        # The fixed protocol version is followed by pairs of parameter name and value strings.
+        # A zero byte is required as a terminator after the last name/value pair.
+        # Parameters can appear in any order.
+        fixed_protocol_version = 3 << 16 | 5
+        bytes_ = pack('!I', fixed_protocol_version)
 
+        # The frontend sends a requested protocol version to the backend.
+        # Old servers (protocol < 3.7) ignore this value and use the fixed protocol version.
+        # New servers (protocol >= 3.7) would try to find the common protocol
+        # version in use for both client and server, and send back a ParameterStatus
+        # message (key='protocol_version', value=<effective protocol version>)
+        bytes_ += pack('!16sxIx', b'protocol_version', vertica_python.PROTOCOL_VERSION)
+
+        for k in self.parameters:
+            v = self.parameters[k].encode('utf-8')
+            bytes_ += pack('!{}sx{}sx'.format(len(k), len(v)), k, v)
+
+        bytes_ += pack('x')
         return bytes_
